@@ -37,22 +37,18 @@ rm -f $basedir/package.xml
 
 rm -fr $targetdir
 mkdir $targetdir
-cp -pR * "$targetdir/"
+cp -a . "$targetdir/"
 
 #  create optional package
 optpkg_dir="$targetdir/misc/optional_package"
-mkdir $optpkg_dir/Smarty/release
 cd $optpkg_dir/Smarty/src
 tar xvfz Smarty*.tar.gz
 cd $optpkg_dir/Smarty/build
-chmod +x ./build
 ./build
 cp $optpkg_dir/Smarty/release/*.tgz $tmpdir
-mkdir $optpkg_dir/simpletest/release
 cd $optpkg_dir/simpletest/src
 tar xvfz simpletest*.tar.gz
 cd $optpkg_dir/simpletest/build
-chmod +x ./build
 ./build
 cp $optpkg_dir/simpletest/release/*.tgz $tmpdir
 rm -rf $optpkg_dir
@@ -65,4 +61,10 @@ php $basedir/bin/ethna_make_package.php $beta
 cp -f $basedir/package.xml $tmpdir/
 cd $tmpdir
 tar zcvf Ethna-$version.tgz package.xml Ethna-$version
-zip -r Ethna-$version.zip package.xml Ethna-$version
+
+cd $basedir
+php $basedir/bin/ethna_make_package.php $beta -o
+cp -f $basedir/package.xml $tmpdir/
+cd $tmpdir
+tar zcvf Ethna-$version-dev.tgz package.xml Ethna-$version
+
