@@ -6,7 +6,7 @@
  *  @author     Masaki Fujimoto <fujimoto@php.net>
  *  @license    http://www.opensource.org/licenses/bsd-license.php The BSD License
  *  @package    Ethna
- *  @version    $Id: Ethna_MailSender.php 537 2008-05-21 22:21:53Z mumumu-org $
+ *  @version    $Id: Ethna_MailSender.php 642 2008-09-19 16:54:53Z mumumu-org $
  */
 
 /** メールテンプレートタイプ: 直接送信 */
@@ -119,6 +119,9 @@ class Ethna_MailSender
                 $template = $this->def[$template];
             }
             $mail = $renderer->perform(sprintf('%s/%s', $this->mail_dir, $template), true);
+            if (Ethna::isError($mail)) {
+                return $mail;
+            }
         }
         if ($to === null) {
             return $mail;
