@@ -6,7 +6,7 @@
  *  @author     ICHII Takashi <ichii386@schweetheart.jp>
  *  @license    http://www.opensource.org/licenses/bsd-license.php The BSD License
  *  @package    Ethna
- *  @version    $Id: Ethna_PearWrapper.php 627 2008-08-04 22:14:08Z mumumu-org $
+ *  @version    $Id: Ethna_PearWrapper.php 701 2009-01-23 18:30:38Z mumumu-org $
  */
 
 require_once 'PEAR.php';
@@ -454,7 +454,7 @@ class Ethna_PearWrapper
         $config =& PEAR_Config::singleton();
         $packagefile =& new PEAR_PackageFile($config);
         $info =& $packagefile->fromTgzFile($filename, PEAR_VALIDATE_NORMAL);
-        if (Ethna::isError($info)) {
+        if (PEAR::isError($info)) {
             return $info;
         }
         $info_array = $info->toArray();
